/* Rexx */

/* Derive an AES EXPORTER key from two ECC key pairs                 */
/*-------------------------------------------------------------------*/
/* Description:                                                      */
/*                                                                   */
/* This REXX contains samples for deriving an AES key from two ECC   */
/* key pairs.                                                        */
/*  - Create a skeleton for the AES EXPORTER key token               */
/*  - Derive the AES key token from the sender's ECC private key and */
/*    the receiver's ECC public key stored in the PKDS               */
/*  - Store the derived AES key token in the CKDS                    */
/*                                                                   */
/* Environment: Sender                                               */
/* Prerequisites: IMPRTECC                                           */
/*                                                                   */
/* How To Run:                                                       */
/* - Execute this script from TSO                                    */
/*   (e.g. EX 'HLQ.MLD.LLQ(DRVAESXP)')                               */
/*-------------------------------------------------------------------*/
signal on novalue

/* Pre-existing key labels in use for this sample */
sender_key_label = LEFT('SAMPLE.SENDER.ECC.KEY',64)
receiver_key_label = LEFT('SAMPLE.RECEIVER.ECC.PUBLIC.KEY',64)

/* Define the key label for the AES exporter key */
exporter_key_label = LEFT('SAMPLE.DERIVED.AES.EXPORTER.KEY',64)

/* CLEANUP the AES key label in use for this sample */
krd_label = exporter_key_label
CALL CSNBKRD

/*********************************************************************/
/* Create a skeleton token for the derived key                       */
/*********************************************************************/
ktb2_rule_array_count  = '00000003'x
ktb2_rule_array        = 'INTERNAL' ||,
                         'AES     ' ||,
                         'EXPORTER'
ktb2_target_key_length = '000002D5'x
ktb2_target_key        = COPIES('00'x,725)
CALL CSNBKTB2

/*********************************************************************/
/* Derive the AES Key                                                */
/*********************************************************************/
edh_rule_array_count      = '00000002'x
edh_rule_array            = 'DERIV01 ' ||,
                            'KEY-AES '
edh_private_key_id_length = '00000040'x
edh_private_key_id        = sender_key_label
edh_public_key_id_length  = '00000040'x
edh_public_key_id         = receiver_key_label
edh_party_id_length       = '0000000E'x
edh_party_id              = 'transaction-id'
edh_key_bit_length        = '00000100'x        /* 256-bit AES  */
edh_output_key_id_length  = '00000384'x        /* 900 byte max */
edh_output_key_id         = LEFT(ktb2_target_key,900,'00'x)
CALL CSNDEDH

/*********************************************************************/
/* Store the AES key in the CKDS                                     */
/*********************************************************************/
krc2_label = exporter_key_label
krc2_token_length = edh_output_key_id_length
krc2_token = SUBSTR(edh_output_key_id,1,140)
CALL CSNBKRC2

krr2_label = exporter_key_label
CALL CSNBKRR2

SAY 'derived AES EXPORTER key label:' exporter_key_label

SAY "-----------------------------------------------------------------"
SAY "End of Sample"
SAY "-----------------------------------------------------------------"

exit

/* --------------------------------------------------------------- */
/* CSNBKTB2 - Key Token Build2                                     */
/*                                                                 */
/* Builds a variable-length CCA symmetric key token which can be   */
/* used as input for the Key Generate2 (CSNBKGN2), Diversified Key */
/* Generate2 (CSNBDKG2) or Secure Key Import2 (CSNBSKI2) callable  */
/* services.                                                       */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKTB2:

ktb2_RC                  = 'FFFFFFFF'x
ktb2_RS                  = 'FFFFFFFF'x
ktb2_exit_data_length    = '00000000'x
ktb2_exit_data           = ''
ktb2_key_bit_length      = '00000000'x
ktb2_key_value           = ''
ktb2_key_name_length     = '00000000'x
ktb2_key_name            = ''
ktb2_user_data_length    = '00000000'x
ktb2_user_data           = ''
ktb2_token_data_length   = '00000000'x
ktb2_token_data          = ''
ktb2_service_data_length = '00000000'x
ktb2_service_data        = ''

ADDRESS linkpgm "CSNBKTB2",
   'ktb2_RC'                      'ktb2_RS'               ,
   'ktb2_exit_data_length'        'ktb2_exit_data'        ,
   'ktb2_rule_array_count'        'ktb2_rule_array'       ,
   'ktb2_key_bit_length'          'ktb2_key_value'        ,
   'ktb2_key_name_length'         'ktb2_key_name'         ,
   'ktb2_user_data_length'        'ktb2_user_data'        ,
   'ktb2_token_data_length'       'ktb2_token_data'       ,
   'ktb2_service_data_length'     'ktb2_service_data'     ,
   'ktb2_target_key_length'       'ktb2_target_key'

IF (ktb2_rc /= '00000000'x) THEN
  DO
    SAY 'KTB Failed   (rc=' C2X(ktb2_rc)' rs='C2X(ktb2_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNDEDH - ECC Diffie-Hellman                                    */
/*                                                                 */
/* Creates symmetric key material from a pair of ECC keys.         */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDEDH:

edh_RC                    = 'FFFFFFFF'x
edh_RS                    = 'FFFFFFFF'x
edh_exit_data_length      = '00000000'x
edh_exit_data             = ''
edh_private_kek_id_length = '00000000'x
edh_private_kek_id        = ''
edh_chain_vector_length   = '00000000'x
edh_chain_vector          = ''
edh_reserved_length       = '00000000'x
edh_reserved              = ''
edh_reserved2_length      = '00000000'x
edh_reserved2             = ''
edh_reserved3_length      = '00000000'x
edh_reserved3             = ''
edh_reserved4_length      = '00000000'x
edh_reserved4             = ''
edh_reserved5_length      = '00000000'x
edh_reserved5             = ''
edh_output_kek_id_length  = '00000000'x
edh_output_kek_id         = ''

ADDRESS linkpgm "CSNDEDH",
   'edh_RC'                          'edh_RS'                     ,
   'edh_exit_data_length'            'edh_exit_data'              ,
   'edh_rule_array_count'            'edh_rule_array'             ,
   'edh_private_key_id_length'       'edh_private_key_id'         ,
   'edh_private_kek_id_length'       'edh_private_kek_id'         ,
   'edh_public_key_id_length'        'edh_public_key_id'          ,
   'edh_chain_vector_length'         'edh_chain_vector'           ,
   'edh_party_id_length'             'edh_party_id'               ,
   'edh_key_bit_length'                                           ,
   'edh_reserved_length'             'edh_reserved'               ,
   'edh_reserved2_length'            'edh_reserved2'              ,
   'edh_reserved3_length'            'edh_reserved3'              ,
   'edh_reserved4_length'            'edh_reserved4'              ,
   'edh_reserved5_length'            'edh_reserved5'              ,
   'edh_output_kek_id_length'        'edh_output_kek_id'          ,
   'edh_output_key_id_length'        'edh_output_key_id'

IF (edh_rc /= '00000000'x) THEN
  DO
    SAY 'edh Failed   (rc=' C2X(edh_rc)' rs='C2X(edh_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNBKRC2 - Key Record Create2                                   */
/*                                                                 */
/* Adds a key token to the CKDS.                                   */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKRC2:
krc2_rc = 'FFFFFFFF'x
krc2_rs = 'FFFFFFFF'x
krc2_exit_length = '00000000'x
krc2_exit_data = ''
krc2_rule_count = '00000000'x
krc2_rule_array = ''

ADDRESS LINKPGM "CSNBKRC2",
   'krc2_rc'                          'krc2_rs'               ,
   'krc2_exit_length'                 'krc2_exit_data'        ,
   'krc2_rule_count'                  'krc2_rule_array'       ,
   'krc2_label'                       'krc2_token_length'     ,
   'krc2_token'

IF (KRC2_RC /= '00000000'x) THEN
  DO
    SAY 'KRC2 Failed   (rc=' C2X(KRC2_RC)' rs='C2X(KRC2_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNBKRR2 - Key Record Read                                      */
/*                                                                 */
/* Reads a key record from the CKDS.                               */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKRR2:
krr2_rc = 'FFFFFFFF'x
krr2_rs = 'FFFFFFFF'x
krr2_exit_length = '00000000'x
krr2_exit_data = ''
krr2_rule_count = '00000000'x
krr2_rule_array = ''
krr2_token_length = '000002D5'x
krr2_token = COPIES('00'x, 725)

ADDRESS LINKPGM "CSNBKRR2",
   'krr2_rc'                          'krr2_rs'               ,
   'krr2_exit_length'                 'krr2_exit_data'        ,
   'krr2_rule_count'                  'krr2_rule_array'       ,
   'krr2_label'                       'krr2_token_length'     ,
   'krr2_token'

IF (krr2_rc /= '00000000'x) THEN
  SAY 'KRR2 Failed   (rc=' C2X(krr2_rc)' rs='C2X(krr2_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* CSNBKRD - Key Record Delete                                     */
/*                                                                 */
/* Deletes a key record from the CKDS.                             */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKRD:
krd_rc = 'FFFFFFFF'x
krd_rs = 'FFFFFFFF'x
krd_exit_length = '00000000'x
krd_exit_data = ''
krd_rule_count = '00000001'x
krd_rule_array = 'LABEL-DL'

ADDRESS LINKPGM "CSNBKRD",
   'krd_rc'                          'krd_rs'               ,
   'krd_exit_length'                 'krd_exit_data'        ,
   'krd_rule_count'                  'krd_rule_array'       ,
   'krd_label'

IF (krd_rc /= '00000000'x & krd_rs /= '0000271C'x) THEN
  SAY 'KRD Failed   (rc=' C2X(krd_rc)' rs='C2X(krd_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* Debug ;-)                                                       */
/* --------------------------------------------------------------- */
NOVALUE:
SAY "Condition NOVALUE was raised."
SAY CONDITION("D") "variable was not initialized."
SAY SOURCELINE(sigl)
EXIT