/* Rexx */

/* Export an AES key under an ECC-derived AES EXPORTER key.          */
/*-------------------------------------------------------------------*/
/* Description:                                                      */
/*                                                                   */
/* This REXX contains samples that:                                  */
/*  - Generate an AES DATA key and store in the CKDS                 */
/*  - Generate a verification pattern for the AES DATA key           */
/*  - Read the key token from the CKDS                               */
/*  - Translate the key token to a variable-length AES CIPHER key    */
/*  - Export the AES CIPHER key under an AES EXPORTER key            */
/*                                                                   */
/* Environment: Sender                                               */
/* Prerequisites: DRVAESXP                                           */
/*                                                                   */
/* Instructions:                                                     */
/* - Execute this script from TSO                                    */
/*   (e.g. EX 'HLQ.MLD.LLQ(EXPAES32)')                               */
/* - Save the output to a text file                                  */
/*-------------------------------------------------------------------*/
signal on novalue

/* Pre-existing key labels in use for this sample */
exporter_key_label = LEFT('SAMPLE.DERIVED.AES.EXPORTER.KEY',64)

/* Define the key label for a random AES data key */
aes_data_key_label = LEFT('ICSF.SECRET.AES256.KEY001',64)

/* CLEANUP labels in use for this sample */
krd_label = aes_data_key_label
CALL CSNBKRD

/*********************************************************************/
/* Generate a 256-bit AES DATA key to export                         */
/*********************************************************************/
kgn_key_form           = 'OP  '
kgn_key_length         = 'KEYLN32 '
kgn_key_type_1         = 'AESDATA '
kgn_key_type_2         = ''
kgn_kek_id_1           = COPIES('00'x,64)
kgn_kek_id_2           = ''
kgn_generated_key_id_1 = COPIES('00'x,64)
kgn_generated_key_id_2 = ''
CALL CSNBKGN

/*********************************************************************/
/* Store the AES DATA key in the CKDS                                */
/*********************************************************************/
krc2_label = aes_data_key_label
krc2_token_length = '00000040'x
krc2_token = kgn_generated_key_id_1
CALL CSNBKRC2

/*********************************************************************/
/* Generate a verification pattern for the AES DATA key              */
/*********************************************************************/
kyt2_rule_count         = '00000003'x
kyt2_rule_array         = 'AES     ' ||,
                          'GENERATE' ||,
                          'SHA-256 '
kyt2_key_id_length      = '00000040'x
kyt2_key_id             = aes_data_key_label
kyt2_ver_pattern_length = '00000008'x
kyt2_ver_pattern        = COPIES('00'x,8)
CALL CSNBKYT2
SAY 'verification pattern:' C2X(kyt2_ver_pattern)

/*********************************************************************/
/* Read the AES DATA key token from the CKDS                         */
/*********************************************************************/
krr2_label = aes_data_key_label
CALL CSNBKRR2

aes_data_key_token_length = krr2_token_length
aes_data_key_token = SUBSTR(krr2_token,1,C2D(krr2_token_length))

/*********************************************************************/
/* Build an AES CIPHER key skeleton token                            */
/*********************************************************************/
ktb2_rule_count = '00000003'x
ktb2_rule_array = 'INTERNAL'||,
                  'AES     '||,
                  'CIPHER  '
CALL CSNBKTB2

aes_cipher_key_skeleton = ,
       SUBSTR(ktb2_target_key_token,1,C2D(ktb2_target_key_token_length))

/*********************************************************************/
/* Translate the AES DATA key to an AES CIPHER key                   */
/*********************************************************************/
ktr2_rule_count = '00000002'x
ktr2_rule_array = 'REFORMAT'||,
                  'AES     '
ktr2_input_key_length = aes_data_key_token_length
ktr2_input_key = aes_data_key_token
ktr2_output_key_length = '00000384'x  /* 900 byte max */
ktr2_output_key = LEFT(aes_cipher_key_skeleton,900)
CALL CSNBKTR2

aes_cipher_key_length = ktr2_output_key_length
aes_cipher_key = ktr2_output_key

/*********************************************************************/
/* Export the AES DATA key under the AES EXPORTER key                */
/*********************************************************************/
syx_rule_array_count      = '00000002'x
syx_rule_array            = 'AES     ' ||,
                            'AESKW   '
syx_source_key_length     = aes_cipher_key_length
syx_source_key            = aes_cipher_key
syx_transport_key_length  = '0000040'x
syx_transport_key         = exporter_key_label
syx_enciphered_key_length = '00000384'x
syx_enciphered_key        = COPIES('00'x,900)
CALL CSNDSYX

exported_aes_key_length = C2D(syx_enciphered_key_length)
exported_aes_key = ,
  C2X(SUBSTR(syx_enciphered_key,1,exported_aes_key_length))
SAY 'exported key: '
CALL parse exported_aes_key, 64
SAY 'exported key LENGTH: ' exported_aes_key_length
SAY 'exported key length (hex): ' D2X(exported_aes_key_length,8)'x'
/* Note: This key must be saved/sent to the receiving system */

SAY '-----------------------------------------------------------------'
SAY 'End of Sample'
SAY '-----------------------------------------------------------------'

EXIT

/* --------------------------------------------------------------- */
/* CSNBKRD - Key Record Delete                                     */
/*                                                                 */
/* Deletes a key record from the CKDS.                             */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKRD:
krd_rc = 'FFFFFFFF'x
krd_rs = 'FFFFFFFF'x
krd_exit_length = '00000000'x
krd_exit_data = ''
krd_rule_count = '00000001'x
krd_rule_array = 'LABEL-DL'

ADDRESS LINKPGM "CSNBKRD",
   'krd_rc'                          'krd_rs'               ,
   'krd_exit_length'                 'krd_exit_length'        ,
   'krd_rule_count'                  'krd_rule_array'       ,
   'krd_label'

IF (krd_rc /= '00000000'x & krd_rs /= '0000271C'x) THEN
  SAY 'KRD Failed   (rc=' C2X(krd_rc)' rs='C2X(krd_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* CSNBKGN - Key Generate                                          */
/*                                                                 */
/* Generates either one or two DES or AES keys encrypted under a   */
/* master key (internal form) or KEK (external form).              */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKGN:

KGN_RC          = 'FFFFFFFF'x
KGN_RS          = 'FFFFFFFF'x
KGN_exit_length = '00000000'x
KGN_exit_data   = ''

ADDRESS linkpgm "CSNBKGN",
   'KGN_RC'                          'KGN_RS'               ,
   'KGN_exit_data_length'            'KGN_exit_data'        ,
   'KGN_key_form'                    'KGN_key_length'       ,
   'KGN_key_type_1'                  'KGN_key_type_2'       ,
   'KGN_kek_id_1'                    'KGN_kek_id_2' ,
   'KGN_generated_key_id_1'          'KGN_generated_key_id_2'

IF (KGN_RC /= '00000000'x) THEN
  DO
    SAY 'KGN Failed   (rc=' C2X(KGN_RC)' rs='C2X(KGN_rs)')'
    EXIT
  END

Return

/* --------------------------------------------------------------- */
/* CSNBKRC2 - Key Record Create2                                   */
/*                                                                 */
/* Adds a key token to the CKDS.                                   */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKRC2:
krc2_rc = 'FFFFFFFF'x
krc2_rs = 'FFFFFFFF'x
krc2_exit_length = '00000000'x
krc2_exit_data = ''
krc2_rule_count = '00000000'x
krc2_rule_array = ''

ADDRESS LINKPGM "CSNBKRC2",
   'krc2_rc'                          'krc2_rs'               ,
   'krc2_exit_length'                 'krc2_exit_data'        ,
   'krc2_rule_count'                  'krc2_rule_array'       ,
   'krc2_label'                       'krc2_token_length'     ,
   'krc2_token'

IF (KRC2_RC /= '00000000'x) THEN
  DO
    SAY 'KRC2 Failed   (rc=' C2X(KRC2_RC)' rs='C2X(KRC2_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNBKTB2 - Key Token Build2                                     */
/*                                                                 */
/* Builds a skeleton key token to be used as input for the key     */
/* generate (CSNBKGN2) service.                                    */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKTB2:

ktb2_RC = 'FFFFFFFF'x
ktb2_RS = 'FFFFFFFF'x
ktb2_exit_data_length = '00000000'x
ktb2_exit_data = ''
ktb2_clear_key_bit_length    = '00000000'x
ktb2_clear_key_value         = ''
ktb2_key_name_length         = '00000000'x
ktb2_key_name                = ''
ktb2_user_assoc_data_length  = '00000000'x
ktb2_user_assoc_data         = ''
ktb2_token_data_length       = '00000000'x
ktb2_token_data              = ''
ktb2_reserved_length         = '00000000'x
ktb2_reserved                = ''
ktb2_target_key_token_length = '00000100'x              /* Decimal 256      */
ktb2_target_key_token = COPIES('00'x,256)

ADDRESS linkpgm "CSNBKTB2",
   'ktb2_rc'                      'ktb2_rs'               ,
   'ktb2_exit_data_length'        'ktb2_exit_data'        ,
   'ktb2_rule_count'              'ktb2_rule_array'       ,
   'ktb2_clear_key_bit_length'    'ktb2_clear_key_value'  ,
   'ktb2_key_name_length'         'ktb2_key_name'         ,
   'ktb2_user_assoc_data_length'  'ktb2_user_assoc_data'  ,
   'ktb2_token_data_length'       'ktb2_token_data'       ,
   'ktb2_reserved_length'         'ktb2_reserved'         ,
   'ktb2_target_key_token_length' 'ktb2_target_key_token'

IF (ktb2_rc /= '00000000'x) THEN
  SAY 'KTB2 Failed   (rc=' C2X(ktb2_RC)' rs='C2X(ktb2_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* CSNBKTR2 - Key Translate                                        */
/*                                                                 */
/* Translates the input key token from one format to another.      */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKTR2:

ktr2_rc                = 'FFFFFFFF'x
ktr2_rs                = 'FFFFFFFF'x
ktr2_exit_data_length  = '00000000'x
ktr2_exit_data         = ''
ktr2_input_kek_length  = '00000000'x
ktr2_input_kek         = ''
ktr2_output_kek_length = '00000000'x
ktr2_output_kek        = ''

ADDRESS linkpgm "CSNBKTR2",
   'ktr2_rc'                   'ktr2_rs'               ,
   'ktr2_exit_data_length'     'ktr2_exit_data'        ,
   'ktr2_rule_count'           'ktr2_rule_array'       ,
   'ktr2_input_key_length'     'ktr2_input_key'        ,
   'ktr2_input_kek_length'     'ktr2_input_kek'        ,
   'ktr2_output_kek_length'    'ktr2_output_kek'       ,
   'ktr2_output_key_length'    'ktr2_output_key'

IF (ktr2_rc /= '00000000'x) THEN
  DO
    SAY 'KTR2 Failed   (rc=' C2X(ktr2_rc)' rs='C2X(ktr2_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNBKRR2 - Key Record Read                                      */
/*                                                                 */
/* Reads a key record from the CKDS.                               */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKRR2:
krr2_rc           = 'FFFFFFFF'x
krr2_rs           = 'FFFFFFFF'x
krr2_exit_length  = '00000000'x
krr2_exit_data    = ''
krr2_rule_count   = '00000000'x
krr2_rule_array   = ''
krr2_token_length = '000002D5'x
krr2_token        = COPIES('00'x, 725)

ADDRESS LINKPGM "CSNBKRR2",
   'krr2_rc'                          'krr2_rs'               ,
   'krr2_exit_length'                 'krr2_exit_data'        ,
   'krr2_rule_count'                  'krr2_rule_array'       ,
   'krr2_label'                       'krr2_token_length'     ,
   'krr2_token'

IF (krr2_rc /= '00000000'x) THEN
  SAY 'KRR2 Failed   (rc=' C2X(krr2_rc)' rs='C2X(krr2_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* CALL CSNBKYT2                                                   */
/*                                                                 */
/* Generate or verify a secure, cryptographic verification pattern */
/* for keys.                                                       */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKYT2:
kyt2_rc              = 'FFFFFFFF'x
kyt2_rs              = 'FFFFFFFF'x
kyt2_exit_length     = '00000000'x
kyt2_exit_data       = ''
kyt2_kek_id_length   = '00000000'x
kyt2_kek_id          = ''
kyt2_reserved_length = '00000000'x
kyt2_reserved        = ''

ADDRESS LINKPGM "CSNBKYT2",
   'kyt2_rc'                   'kyt2_rs'               ,
   'kyt2_exit_length'          'kyt2_exit_data'        ,
   'kyt2_rule_count'           'kyt2_rule_array'       ,
   'kyt2_key_id_length'        'kyt2_key_id'           ,
   'kyt2_kek_id_length'        'kyt2_kek_id'           ,
   'kyt2_reserved_length'      'kyt2_reserved'         ,
   'kyt2_ver_pattern_length'   'kyt2_ver_pattern'

IF (kyt2_rc /= '00000000'x) THEN
  SAY 'KYT2 Failed   (rc=' C2X(kyt2_rc)' rs='C2X(kyt2_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* CSNDSYX - Symmetric Key Export                                  */
/*                                                                 */
/* Transfer an application-supplied AES, DES or variable-length    */
/* symmetric key token key from encryption under a master key to   */
/* encryption under an application-supplied RSA public key or AES  */
/* EXPORTER key.                                                   */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDSYX:
syx_rc = 'FFFFFFFF'x
syx_rs = 'FFFFFFFF'x
syx_exit_length = '00000000'x
syx_exit_data = ''

ADDRESS linkpgm "CSNDSYX",
   'syx_rc'                          'syx_rs'               ,
   'syx_exit_length'                 'syx_exit_data'        ,
   'syx_rule_array_count'            'syx_rule_array'       ,
   'syx_source_key_length'           'syx_source_key'       ,
   'syx_transport_key_length'        'syx_transport_key'    ,
   'syx_enciphered_key_length'       'syx_enciphered_key'

IF (syx_rc /= '00000000'x) THEN
  SAY 'SYX Failed   (rc=' C2X(syx_rc)' rs='C2X(syx_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* Parse                                                           */
/*                                                                 */
/* Helper routine to display hex data with a fixed line length     */
/* --------------------------------------------------------------- */
Parse:
ARG data, max

/* The maximum length of an output line */
line_length = max
data_length = LENGTH(data)
num_lines = data_length % line_length

/* Parse the data */
IF data_length // line_length <> 0 THEN num_lines = num_lines + 1
index = 1
DO num_lines
   SAY SUBSTR(data,index,line_length)
   index = index + line_length
END

RETURN

/* --------------------------------------------------------------- */
/* Debug ;-)                                                       */
/* --------------------------------------------------------------- */
NOVALUE:
SAY "Condition NOVALUE was raised."
SAY CONDITION("D") "variable was not initialized."
SAY SOURCELINE(sigl)
EXIT