/* Rexx */

/* Generate an ECC key to derive an AES IMPORTER/EXPORTER key        */
/*-------------------------------------------------------------------*/
/* Description:                                                      */
/*                                                                   */
/* This REXX contains samples for generating an ECC key pair.        */
/*  - Generate an ECC key pair                                       */
/*                                                                   */
/* Environment: Sender or Receiver                                   */
/*                                                                   */
/* How To Run:                                                       */
/* - Update the key label to SENDER or RECEIVER                      */
/* - Execute this script from TSO                                    */
/*   (e.g. EX 'HLQ.MLD.LLQ(GENECC2)')                                */
/* - Save the output (ECC public key) to a text file                 */
/*-------------------------------------------------------------------*/
signal on novalue

/* Define the key label for the current system */
ecc_key_label = LEFT('SAMPLE.SENDER.ECC.PRIVATE.KEY',64)

/* CLEANUP labels in use for this sample */
krd_label = ecc_key_label
Call CSNDKRD

/*********************************************************************/
/* Create a record for the ECC Key Pair in the PKDS                  */
/*********************************************************************/
krc_label        = ecc_key_label
krc_token_length = '00000000'x
krc_token        = ''
Call CSNDKRC

/*********************************************************************/
/* Build the skeleton token for the Key Pair                         */
/*********************************************************************/
pkb_rule_count                 = '00000002'x
pkb_rule_array                 = 'ECC-PAIR' ||,
                                 'KEY-MGMT'
pkb_key_value_structure_length = '00000008'x
pkb_key_value_structure        = '00'x   ||,  /* Prime curve         */
                                 '00'x   ||,  /* reserved            */
                                 '0209'x ||,  /* P-521 curve in bits */
                                 '0000'x ||,  /* length of d in bytes*/
                                 '0000'x      /* public key q, bytes */
pkb_private_key_name_length    = '00000040'x
pkb_private_key_name           = ecc_key_label
pkb_key_token_length           = '00000DAC'x
pkb_key_token                  = COPIES('00'x,3500)
Call CSNDPKB

/*********************************************************************/
/* Generate the Key Pair from the skeleton. Store in the PKDS.       */
/*********************************************************************/
pkg_rule_count                     = '00000001'x
pkg_rule_array                     = 'MASTER  '
pkg_regeneration_data_length       = '00000000'x
pkg_regeneration_data              = ''
pkg_skeleton_key_id_length         = pkb_key_token_length
pkg_skeleton_key_id                = pkb_key_token
pkg_transport_key_identifier       = COPIES('00'x,64)
pkg_generated_key_token_length     = '00000040'x
pkg_generated_key_token            = ecc_key_label
Call CSNDPKG

/*********************************************************************/
/* Extract the public key from the private key                       */
/*********************************************************************/
PKX_source_key_identifier_length = '00000040'x
PKX_source_key_identifier = ecc_key_label
PKX_target_public_key_length = '00000DAC'x /* 3500 decimal max */
PKX_target_public_key = copies('00'x,3500)
Call CSNDPKX

ecc_pubkey_length = PKX_target_public_key_length
ecc_pubkey = PKX_target_public_key
ecc_pubkey = substr(ecc_pubkey,1,c2d(ecc_pubkey_length))

SAY "ecc public key length (hex)" c2X(ecc_pubkey_length)'x'
SAY "ecc public key "
CALL parse C2X(ecc_pubkey), 64

SAY "-----------------------------------------------------------------"
SAY "End of Sample"
SAY "-----------------------------------------------------------------"

EXIT

/* --------------------------------------------------------------- */
/* CSNDPKB - PKA Key Token Build                                   */
/*                                                                 */
/* Builds a skeleton key token which can be used as input for the  */
/* key generate (CSNDPKG) callable service.                        */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDPKB:

PKB_RC = 'FFFFFFFF'x
PKB_RS = 'FFFFFFFF'x
PKB_exit_data_length  = '00000000'x
PKB_exit_data  = ''
PKB_reserved_1_length = '00000000'x
PKB_reserved_1 = ''
PKB_reserved_2_length = '00000000'x
PKB_reserved_2 = ''
PKB_reserved_3_length = '00000000'x
PKB_reserved_3 = ''
PKB_reserved_4_length = '00000000'x
PKB_reserved_4 = ''
PKB_reserved_5_length = '00000000'x
PKB_reserved_5 = ''

ADDRESS linkpgm "CSNDPKB",
   'PKB_RC'                         'PKB_RS'                     ,
   'PKB_exit_data_length'           'PKB_exit_data'              ,
   'PKB_rule_count'                 'PKB_rule_array'             ,
   'PKB_key_value_structure_length' 'PKB_key_value_structure'    ,
   'PKB_private_key_name_length'    'PKB_private_key_name'       ,
   'PKB_reserved_1_length'          'PKB_reserved_1'             ,
   'PKB_reserved_2_length'          'PKB_reserved_2'             ,
   'PKB_reserved_3_length'          'PKB_reserved_3'             ,
   'PKB_reserved_4_length'          'PKB_reserved_4'             ,
   'PKB_reserved_5_length'          'PKB_reserved_5'             ,
   'PKB_key_token_length'           'PKB_key_token'

IF (PKB_RC /= '00000000'x) Then
  DO
    SAY 'PKB Failed   (rc=' C2X(PKB_RC)' rs='C2X(PKB_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNDPKG - PKA Key Generate                                      */
/*                                                                 */
/* Generates RSA or ECC key pairs.                                 */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDPKG:

PKG_rc          = 'FFFFFFFF'x
PKG_rs          = 'FFFFFFFF'x
PKG_exit_length = '00000000'x
PKG_exit_data   = ''

ADDRESS linkpgm "CSNDPKG",
   'PKG_rc'                          'PKG_rs'                 ,
   'PKG_exit_length'                 'PKG_exit_data'          ,
   'PKG_rule_count'                  'PKG_rule_array'         ,
   'PKG_regeneration_data_length'    'PKG_regeneration_data'  ,
   'PKG_skeleton_key_id_length'      'PKG_skeleton_key_id'    ,
   'PKG_transport_key_identifier'                             ,
   'PKG_generated_key_token_length'  'PKG_generated_key_token'

IF (PKG_RC /= '00000000'x) Then
  DO
    SAY 'PKG Failed   (rc=' C2X(PKG_RC)' rs='C2X(PKG_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNDPKX - PKA Public Key Extract                                */
/*                                                                 */
/* Extracts the public key from a supplied private key.            */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDPKX:

PKX_rc          = 'FFFFFFFF'x
PKX_rs          = 'FFFFFFFF'x
PKX_exit_length = '00000000'x
PKX_exit_data   = ''
PKX_rule_array_count = '00000000'x
PKX_rule_array  = ''

ADDRESS linkpgm "CSNDPKX",
   'PKX_rc'                           'PKX_rs'                    ,
   'PKX_exit_length'                  'PKX_exit_data'             ,
   'PKX_rule_array_count'             'PKX_rule_array'            ,
   'PKX_source_key_identifier_length' 'PKX_source_key_identifier' ,
   'PKX_target_public_key_length'     'PKX_target_public_key'     ;

IF (PKX_RC /= '00000000'x) Then
  DO
    SAY 'PKX Failed   (rc=' C2X(PKX_RC)' rs='C2X(PKX_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNDKRC - Key Record Create (PKDS)                              */
/*                                                                 */
/* Adds a key record and key token to the PKDS.                    */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDKRC:
krc_rc          = 'FFFFFFFF'x
krc_rs          = 'FFFFFFFF'x
krc_exit_length = '00000000'x
krc_exit_data   = ''
krc_rule_count  = '00000000'x
krc_rule_array  = ''

ADDRESS LINKPGM "CSNDKRC",
   'krc_rc'                          'krc_rs'               ,
   'krc_exit_length'                 'krc_exit_data'        ,
   'krc_rule_count'                  'krc_rule_array'       ,
   'krc_label'                                              ,
   'krc_token_length'                'krc_token'

IF (krc_rc /= '00000000'x) THEN
  DO
    SAY 'KRC Failed   (rc=' C2X(krc_rc)' rs='C2X(krc_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNDKRR - Key Record Read (PKDS)                                */
/*                                                                 */
/* Reads a key token from the PKDS.                                */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDKRR:
krr_rc          = 'FFFFFFFF'x
krr_rs          = 'FFFFFFFF'x
krr_exit_length = '00000000'x
krr_exit_data   = ''
krr_rule_count  = '00000000'x
krr_rule_array  = ''

ADDRESS LINKPGM "CSNDKRR",
   'krr_rc'                          'krr_rs'               ,
   'krr_exit_length'                 'krr_exit_data'        ,
   'krr_rule_count'                  'krr_rule_array'       ,
   'krr_label'                                              ,
   'krr_token_length'                'krr_token'

IF (krr_RC /= '00000000'x) THEN
  DO
    SAY 'KRR Failed   (rc=' C2X(krr_rc)' rs='C2X(krr_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNDKRD - Key Record Delete (PKDS)                              */
/*                                                                 */
/* Deletes a key record from the PKDS.                             */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDKRD:
krd_rc          = 'FFFFFFFF'x
krd_rs          = 'FFFFFFFF'x
krd_exit_length = '00000000'x
krd_exit_data   = ''
krd_rule_count  = '00000001'x
krd_rule_array  = 'LABEL-DL'

ADDRESS LINKPGM "CSNDKRD",
   'krd_rc'                          'krd_rs'               ,
   'krd_exit_length'                 'krd_exit_data'        ,
   'krd_rule_count'                  'krd_rule_array'       ,
   'krd_label'

IF (krd_rc /= '00000000'x & krd_rs /= '0000271C'x) THEN
  SAY 'KRD Failed   (rc=' C2X(krd_rc)' rs='C2X(krd_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* Parse                                                           */
/*                                                                 */
/* Helper routine to display hex data with a fixed line length     */
/* --------------------------------------------------------------- */
Parse:
ARG data, max

/* The maximum length of an output line */
line_length = max
data_length = LENGTH(data)
num_lines = data_length % line_length

/* Parse the data */
IF data_length // line_length <> 0 THEN num_lines = num_lines + 1
index = 1
DO num_lines
   SAY SUBSTR(data,index,line_length)
   index = index + line_length
END

RETURN

/* --------------------------------------------------------------- */
/* Debug ;-)                                                       */
/* --------------------------------------------------------------- */
NOVALUE:
SAY "Condition NOVALUE was raised."
SAY CONDITION("D") "variable was not initialized."
SAY SOURCELINE(sigl)
EXIT