/* Rexx */

/* Import an AES key wrapped using an ECC-derived AES IMPORTER key.  */
/*-------------------------------------------------------------------*/
/* Description:                                                      */
/*                                                                   */
/* This REXX contains samples that:                                  */
/*  - Export an AES CIPHER key using an AES IMPORTER key             */
/*  - Translate the AES CIPHER KEY to an AES DATA key                */
/*  - Store the AES DATA key in the CKDS                             */
/*  - Verify the AES DATA key                                        */
/*                                                                   */
/* Environment: Receiver                                             */
/* Prerequisites: DRVAESMP and EXPAESKW (Sender)                     */
/*                                                                   */
/* Instructions:                                                     */
/* - Update the values for encrypted_key_length, encrypted key and   */
/*    key verification pattern                                       */
/* - Execute this script from TSO                                    */
/*   (e.g. EX 'HLQ.MLD.LLQ(IMPAES32)')                               */
/*-------------------------------------------------------------------*/
signal on novalue

/* Pre-existing key labels in use for this sample */
importer_key_label = LEFT('SAMPLE.DERIVED.AES.IMPORTER.KEY',64)

/* Define the key label for the received AES data key */
aes_data_key_label = LEFT('SAMPLE.RECEIVED.AES.DATA.KEY',64)

/* Specify data from the sender */
verification_pattern = 'D44130237A4A57A1'x
encrypted_key = ,
'02000088050000000202CB23932FB20B74190000000000000000020201000100'x||,
'001A0000000002800002000102C000000003E0000000001464DC21204EBF908C'x||,
'45B4BE71F210A12A4934941E8AA74EB33CF44695C8ADBB0C0F05DB13E31DB706'x||,
'401F1CE1479B57B0B08CB8192B984E847BF48E82BAEDD7FCC1D4D6B82E3EFDC7'x||,
'F2C89BEC5AD7758F'x
encrypted_key_length = '00000088'x

/* CLEANUP the AES key label in use for this sample */
krd_label = aes_data_key_label
CALL CSNBKRD

/*********************************************************************/
/* Import the encrypted key using the AES IMPORTER key               */
/*********************************************************************/
syi2_rule_count            = '00000002'x
syi2_rule_array            = 'AES     ' ||,
                             'AESKW   '
syi2_enciphered_key_length = encrypted_key_length
syi2_enciphered_key        = encrypted_key
syi2_transport_key_length  = '00000040'x
syi2_transport_key         = importer_key_label
syi2_target_key_length     = '000002D5'x
syi2_target_key            = copies('00'x,725)
CALL CSNDSYI2

/*********************************************************************/
/* Translate the AES CIPHER key to an AES DATA key                   */
/*********************************************************************/
ktr2_rule_count        = '00000002'x
ktr2_rule_array        = 'REFORMAT'||,
                         'AES     '
ktr2_input_key_length  = syi2_target_key_length
ktr2_input_key         = syi2_target_key
ktr2_output_key_length = '00000384'x  /* 900 byte max */
ktr2_output_key        = copies('00'x,900)
CALL CSNBKTR2

/*********************************************************************/
/* Store the AES DATA key in the CKDS                                */
/*********************************************************************/
krc2_label        = aes_data_key_label
krc2_token_length = ktr2_output_key_length
krc2_token        = ktr2_output_key
CALL CSNBKRC2

/*********************************************************************/
/* Verify the AES DATA key                                           */
/*********************************************************************/
kyt2_rule_count         = '00000003'x
kyt2_rule_array         = 'AES     ' ||,
                          'GENERATE' ||,
                          'SHA-256 '
kyt2_key_id_length      = '00000040'x
kyt2_key_id             = aes_data_key_label
kyt2_ver_pattern_length = '00000008'x
kyt2_ver_pattern        = verification_pattern
CALL CSNBKYT2

IF kyt2_rc = '00000000'x THEN
  SAY 'Verification of the AES DATA KEY succeeded'
ELSE
  SAY 'Verification of the AES DATA KEY failed'

SAY '-----------------------------------------------------------------'
SAY 'End of Sample'
SAY '-----------------------------------------------------------------'

EXIT

/* --------------------------------------------------------------- */
/* CSNDSYI - Symmetric Key Import2                                 */
/*                                                                 */
/* Transfers an HMAC, AES or DES key enciphered under an RSA public*/
/* key or AES EXPORTER key to encryption under a master key.       */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDSYI2:

syi2_rc               = 'FFFFFFFF'x
syi2_rs               = 'FFFFFFFF'x
syi2_exit_length      = '00000000'x
syi2_exit_data        = ''
syi2_key_name_length  = '00000000'x
syi2_key_name         = ''

ADDRESS linkpgm "CSNDSYI2",
   'syi2_rc'                          'syi2_rs'                 ,
   'syi2_exit_length'                 'syi2_exit_data'          ,
   'syi2_rule_count'                  'syi2_rule_array'         ,
   'syi2_enciphered_key_length'       'syi2_enciphered_key'     ,
   'syi2_transport_key_length'        'syi2_transport_key'      ,
   'syi2_key_name_length'             'syi2_key_name'           ,
   'syi2_target_key_length'           'syi2_target_key'

IF (syi2_rc /= '00000000'x) THEN
  SAY 'SYI2 Failed   (rc=' C2X(syi2_rc)' rs='C2X(syi2_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* CSNBKTR2 - Key Translate                                        */
/*                                                                 */
/* Translates the input key token from one format to another.      */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKTR2:

ktr2_rc                = 'FFFFFFFF'x
ktr2_rs                = 'FFFFFFFF'x
ktr2_exit_data_length  = '00000000'x
ktr2_exit_data         = ''
ktr2_input_kek_length  = '00000000'x
ktr2_input_kek         = ''
ktr2_output_kek_length = '00000000'x
ktr2_output_kek        = ''

ADDRESS linkpgm "CSNBKTR2",
   'ktr2_rc'                   'ktr2_rs'               ,
   'ktr2_exit_data_length'     'ktr2_exit_data'        ,
   'ktr2_rule_count'           'ktr2_rule_array'       ,
   'ktr2_input_key_length'     'ktr2_input_key'        ,
   'ktr2_input_kek_length'     'ktr2_input_kek'        ,
   'ktr2_output_kek_length'    'ktr2_output_kek'       ,
   'ktr2_output_key_length'    'ktr2_output_key'

IF (ktr2_rc /= '00000000'x) THEN
  DO
    SAY 'KTR2 Failed   (rc=' C2X(ktr2_rc)' rs='C2X(ktr2_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNBKRC2 - Key Record Create2                                   */
/*                                                                 */
/* Adds a key token to the CKDS.                                   */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKRC2:
krc2_rc = 'FFFFFFFF'x
krc2_rs = 'FFFFFFFF'x
krc2_exit_data_length = '00000000'x
krc2_exit_data = ''
krc2_rule_count = '00000000'x
krc2_rule_array = ''

ADDRESS LINKPGM "CSNBKRC2",
   'krc2_rc'                   'krc2_rs'               ,
   'krc2_exit_length'          'krc2_exit_data'        ,
   'krc2_rule_count'           'krc2_rule_array'       ,
   'krc2_label'                                        ,
   'krc2_token_length'         'krc2_token'

IF (krc2_rc /= '00000000'x) Then
  DO
    SAY 'krc2 Failed   (rc=' C2X(krc2_rc)' rs='C2X(krc2_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNBKRD - Key Record Delete                                     */
/*                                                                 */
/* Deletes a key record from the CKDS.                             */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKRD:
krd_rc = 'FFFFFFFF'x
krd_rs = 'FFFFFFFF'x
krd_exit_length = '00000000'x
krd_exit_data = ''
krd_rule_count = '00000001'x
krd_rule_array = 'LABEL-DL'

ADDRESS LINKPGM "CSNBKRD",
   'krd_rc'                          'krd_rs'               ,
   'krd_exit_length'                 'krd_exit_data'        ,
   'krd_rule_count'                  'krd_rule_array'       ,
   'krd_label'

IF (krd_rc /= '00000000'x & krd_rs /= '0000271C'x) THEN
  SAY 'KRD Failed   (rc=' C2X(krd_rc)' rs='C2X(krd_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* CALL CSNBKYT2                                                   */
/*                                                                 */
/* Generate or verify a secure, cryptographic verification pattern */
/* for keys.                                                       */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNBKYT2:
kyt2_rc              = 'FFFFFFFF'x
kyt2_rs              = 'FFFFFFFF'x
kyt2_exit_length     = '00000000'x
kyt2_exit_data       = ''
kyt2_kek_id_length   = '00000000'x
kyt2_kek_id          = ''
kyt2_reserved_length = '00000000'x
kyt2_reserved        = ''

ADDRESS LINKPGM "CSNBKYT2",
   'kyt2_rc'                   'kyt2_rs'               ,
   'kyt2_exit_length'          'kyt2_exit_data'        ,
   'kyt2_rule_count'           'kyt2_rule_array'       ,
   'kyt2_key_id_length'        'kyt2_key_id'           ,
   'kyt2_kek_id_length'        'kyt2_kek_id'           ,
   'kyt2_reserved_length'      'kyt2_reserved'         ,
   'kyt2_ver_pattern_length'   'kyt2_ver_pattern'

IF (kyt2_rc /= '00000000'x) THEN
  SAY 'KYT2 Failed   (rc=' C2X(kyt2_rc)' rs='C2X(kyt2_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* Debug ;-)                                                       */
/* --------------------------------------------------------------- */
NOVALUE:
SAY "Condition NOVALUE was raised."
SAY CONDITION("D") "variable was not initialized."
SAY SOURCELINE(sigl)
EXIT