/* Rexx */

/* Sample: Create an ECC public key from a partner's public key      */
/*-------------------------------------------------------------------*/
/* Description:                                                      */
/*                                                                   */
/* This REXX contains samples for building an ECC public key:        */
/*  - Create an ECC public key an existing CCA key token             */
/*  - Store the key in the PKDS                                      */
/*                                                                   */
/* Environment: Sender or Receiver                                   */
/* Prerequisites: GENECC2                                            */
/*                                                                   */
/* How To Run:                                                       */
/* - Update the key label to indicate your partner's public key      */
/* - Update ecc_pubkey and ecc_pubkey_length from your partner's     */
/*   ECC public key                                                  */
/* - Execute this script from TSO                                    */
/*   (e.g. EX 'HLQ.MLD.LLQ(IMPRTEC2)')                               */
/*-------------------------------------------------------------------*/
signal on novalue

/* Define the key label for the partner's key */
ecc_pubkey_label = LEFT('SAMPLE.SENDER.ECC.PUBLIC.KEY',64)

/* Specify the partner's ECC public key token */
ecc_pubkey = ,
'1E00009B0000000021000093000000000000020900850401CD05B0FF027DE5B6'x||,
'29EA35FB448E876BC01BBDB007A39C3AE3E2E9D1DC4F524402092FC96D0FB4E8'x||,
'B90F088F4C3C96949DA9D9C9904CF9996D0F0A1260FB2747D6019984209AD80B'x||,
'13ED347FE5CBD2C6B070FE81771B9086DF0AFC8720D9D322F47579FC507D487E'x||,
'EC90968CB7E66673F2C2CC9060AA2170AFBC4B7160AA978BC66E1D'x
ecc_pubkey_length = '0000009B'x

/* CLEANUP labels in use for this sample */
krd_label = ecc_pubkey_label
CALL CSNDKRD

/*********************************************************************/
/* Create a record for the ECC Public Key in the PKDS                */
/*********************************************************************/
krc_label        = ecc_pubkey_label
krc_token        = ecc_pubkey
krc_token_length = ecc_pubkey_length
CALL CSNDKRC

/*********************************************************************/
/* Read the Public Key from the PKDS                                 */
/*********************************************************************/
krr_label        = ecc_pubkey_label
krr_token_length = '00000DAC'x
krr_token        = COPIES('00'x, 3500)
CALL CSNDKRR

ecc_pubkey_length = C2D(krr_token_length)
ecc_pubkey = SUBSTR(krr_token,1,ecc_pubkey_length)

SAY "ecc key label " ecc_pubkey_label
SAY "ecc public key "
CALL parse C2X(ecc_pubkey), 64
SAY "ecc public key length" ecc_pubkey_length
SAY "ecc public key length (hex)" D2X(ecc_pubkey_length,8)'x'

SAY '-----------------------------------------------------------------'
SAY 'End of Sample'
SAY '-----------------------------------------------------------------'

exit

/* --------------------------------------------------------------- */
/* CSNDPKB - PKA Key Token Build                                   */
/*                                                                 */
/* Creates PKA key tokens.                                         */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDPKB:

PKB_RC = 'FFFFFFFF'x
PKB_RS = 'FFFFFFFF'x
PKB_exit_data_length         = '00000000'x
PKB_exit_data                = ''
PKB_private_key_name_length  = '00000000'x
PKB_private_key_name         = ''
PKB_user_assoc_data_length   = '00000000'x
PKB_user_assoc_data          = ''
PKB_reserved_2_length        = '00000000'x
PKB_reserved_2               = ''
PKB_reserved_3_length        = '00000000'x
PKB_reserved_3               = ''
PKB_reserved_4_length        = '00000000'x
PKB_reserved_4               = ''
PKB_reserved_5_length        = '00000000'x
PKB_reserved_5               = ''

ADDRESS linkpgm "CSNDPKB",
   'PKB_RC'                          'PKB_RS'               ,
   'PKB_exit_data_length'            'PKB_exit_data'        ,
   'PKB_rule_count'                  'PKB_rule_array'       ,
   'PKB_kvs_length'                  'PKB_kvs'              ,
   'PKB_private_key_name_length'     'PKB_private_key_name' ,
   'PKB_user_assoc_data_length'      'PKB_user_assoc_data'  ,
   'PKB_reserved_2_length'           'PKB_reserved_2'       ,
   'PKB_reserved_3_length'           'PKB_reserved_3'       ,
   'PKB_reserved_4_length'           'PKB_reserved_4'       ,
   'PKB_reserved_5_length'           'PKB_reserved_5'       ,
   'PKB_key_token_length'            'PKB_key_token'

IF (PKB_RC /= '00000000'x) THEN
  SAY 'PKB Failed   (rc=' C2X(PKB_RC)' rs='C2X(PKB_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* CSNDKRC - Key Record Create (PKDS)                              */
/*                                                                 */
/* Adds a key record and key token to the PKDS.                    */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDKRC:
krc_rc          = 'FFFFFFFF'x
krc_rs          = 'FFFFFFFF'x
krc_exit_length = '00000000'x
krc_exit_data   = ''
krc_rule_count  = '00000000'x
krc_rule_array  = ''

ADDRESS LINKPGM "CSNDKRC",
   'krc_rc'                          'krc_rs'               ,
   'krc_exit_length'                 'krc_exit_data'        ,
   'krc_rule_count'                  'krc_rule_array'       ,
   'krc_label'                                              ,
   'krc_token_length'                'krc_token'

IF (krc_rc /= '00000000'x) THEN
  DO
    SAY 'KRC Failed   (rc=' C2X(krc_rc)' rs='C2X(krc_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNDKRR - Key Record Read (PKDS)                                */
/*                                                                 */
/* Reads a key token from the PKDS.                                */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDKRR:
krr_rc          = 'FFFFFFFF'x
krr_rs          = 'FFFFFFFF'x
krr_exit_length = '00000000'x
krr_exit_data   = ''
krr_rule_count  = '00000000'x
krr_rule_array  = ''

ADDRESS LINKPGM "CSNDKRR",
   'krr_rc'                          'krr_rs'               ,
   'krr_exit_length'                 'krr_exit_data'        ,
   'krr_rule_count'                  'krr_rule_array'       ,
   'krr_label'                                              ,
   'krr_token_length'                'krr_token'

IF (krr_RC /= '00000000'x) THEN
  DO
    SAY 'KRR Failed   (rc=' C2X(krr_rc)' rs='C2X(krr_rs)')'
    EXIT
  END

RETURN

/* --------------------------------------------------------------- */
/* CSNDKRD - Key Record Delete (PKDS)                              */
/*                                                                 */
/* Deletes a key record from the PKDS.                             */
/*                                                                 */
/* See the ICSF Application Programmer's Guide for more details.   */
/* --------------------------------------------------------------- */
CSNDKRD:
krd_rc          = 'FFFFFFFF'x
krd_rs          = 'FFFFFFFF'x
krd_exit_length = '00000000'x
krd_exit_data   = ''
krd_rule_count  = '00000001'x
krd_rule_array  = 'LABEL-DL'

ADDRESS LINKPGM "CSNDKRD",
   'krd_rc'                          'krd_rs'               ,
   'krd_exit_length'                 'krd_exit_data'        ,
   'krd_rule_count'                  'krd_rule_array'       ,
   'krd_label'

IF (krd_rc /= '00000000'x & krd_rs /= '0000271C'x) THEN
  SAY 'KRD Failed   (rc=' C2X(krd_rc)' rs='C2X(krd_rs)')'

RETURN

/* --------------------------------------------------------------- */
/* Parse                                                           */
/*                                                                 */
/* Helper routine to display hex data with a fixed line length     */
/* --------------------------------------------------------------- */
Parse:
ARG data, max

/* The maximum length of an output line */
line_length = max
data_length = LENGTH(data)
num_lines = data_length % line_length

/* Parse the data */
IF data_length // line_length <> 0 THEN num_lines = num_lines + 1
index = 1
DO num_lines
   SAY SUBSTR(data,index,line_length)
   index = index + line_length
END

RETURN

/* --------------------------------------------------------------- */
/* Debug ;-)                                                       */
/* --------------------------------------------------------------- */
NOVALUE:
SAY "Condition NOVALUE was raised."
SAY CONDITION("D") "variable was not initialized."
SAY SOURCELINE(sigl)
EXIT